"""
This script calculates the LD between significant interactions, and groups together redundant pairs.
"""

import csv
import argparse
import subprocess
import sys
import glob
import os

global gene_redundant_pairs_dict
gdict = {}


##################################################################################################
parser = argparse.ArgumentParser(description = "This function uses LD to identify redundant interactions. In order to be considered redundant both SNPs in one interactions must be in LD with at least one" +
								" SNP in the other interaction, and vice versa.")

parser.add_argument('-signif_int', '--input_interactions_file', help = "Specify the file containing significant interactions. This file is produced by the 'get_significant_interactions.py' script." +
					" It also can take the filtered output from calculate_within_pair_ld.py script. This is required", 
					action='store', required = True)
parser.add_argument('-ped_dir', '--input_pedmap_directory', help = "Specify the directory that contains the PED/MAP files used to run the INTERSNP analysis. This script assumes that the naming convention is dirctory_name/GeneID.ped." +
					" This is required", action='store', required = True)
parser.add_argument('-rsquare', '--r2_threshold', help = "Specify the minimum r2 necessary between variants in order to be considered redundant. Ex: 0.9; Default = 1.1", 
					action='store', required = False, default = 1.1)
parser.add_argument('-output_name', '--output_results', help = "Specify the name of the results file. This is required.", 
					action='store', required = True)
parser.add_argument('-ld_dir', '--output_ld_directory', help = "Specify the directory in which to put the LD files that PLINK will generate. The final blackslash should be included (i.e., directory_name/)." +
					" This is required", 
					action='store', required = True)
parser.add_argument('--ppart',type=int,default=1,help="Number of partition")
parser.add_argument('--ppidx',type=int,default=0,help='Index of partition to process')

args = parser.parse_args()

#recreate the command given
command = ""
for com in sys.argv:
	command = command + str(com) + " "

#create the logfile
logfile_name = "%s[%d].log"%(args.output_results[:-4],args.ppidx)
logfile = open(logfile_name, 'w')
logfile.write("You ran the command:\n")
logfile.write(command + "\n")

print "You ran the command:"
print command

#check input files can be found.
input_files = [args.input_interactions_file]
for input_file in input_files:
	if not os.path.isfile(input_file):
		logfile.write("Cannot find file " + str(input_file))
		parser.error("Cannot find file " + str(input_file))

#####################################################################################################################
#######   This section of code builds a dictionary that basically just contains a list of all SNPS
#######   that were identified as significantly interacting for each gene.
#####################################################################################################################

print "Identifying all significant SNPs for each gene with significant interactions."
logfile.write("Identifying all significant SNPs for each gene with significant interactions.\n")

fin_signif_interactions = open(args.input_interactions_file, 'rU')
int_reader = csv.reader(fin_signif_interactions, delimiter = '\t')

snps_to_genes_dict = {} #key is geneID, values are all SNPs identified as significant for that gene

count_sig_int = 0

for row in int_reader:
	gene_id = row[0]
	snp1    = row[3]
	snp2    = row[7]

	if gene_id not in snps_to_genes_dict.keys():
		snps_to_genes_dict[gene_id] = [snp1, snp2]
	elif gene_id in snps_to_genes_dict.keys():
		if snp1 not in snps_to_genes_dict[gene_id]:
			snps_to_genes_dict[gene_id].append(snp1)
		if snp2 not in snps_to_genes_dict[gene_id]:
			snps_to_genes_dict[gene_id].append(snp2)

	count_sig_int = count_sig_int + 1
fin_signif_interactions.close()

print "DONE: Identifying all significant SNPs for each gene with significant interactions."
logfile.write("DONE: Identifying all significant SNPs for each gene with significant interactions.\n")

print "A total of " + str(count_sig_int) + " interactions were read in."
logfile.write("A total of " + str(count_sig_int) + " interactions were read in.\n")

print "\nIdentifying genes in partition %d of %d for subset processing..."%(args.ppidx,args.ppart)
logfile.write("Identifying genes in partition %d of %d for subset processing..."%(args.ppidx,args.ppart))
# Order genes by the number of SNPs
gene_set  = [(len(snps),gene) for gene,snps in snps_to_genes_dict.items()]
gene_set  = sorted(gene_set)
gene_part = {}
# Evenly distribute gene sizes across partitions
for i,(size,gene) in enumerate(gene_set):
	if i%args.ppart == args.ppidx:
		# Retain genes assigned to this partition
		gene_part[gene] = snps_to_genes_dict[gene]
# Overwrite the original dictionary, partition ready for processing
snps_to_genes_dict = gene_part
print "Partition has been populated (%d genes) and is ready for parallel processing.\n"%(len(snps_to_genes_dict))
logfile.write("Partition has been populated and is ready for parallel processing.")

############################################################################################

def calc_between_pair_ld(snps_to_genes_dict):
	#####################################################################################################################
	#######   This section of code builds uses the dictionary built above to create all pairs of variants for a given gene.
	#######   The LD between those variants is then computed using PLINK.
	#######	  Each variant is assigned as being in perfect LD with itself.
	#######	  Only variants in LD above the set r2 threshold are retained in the gene_ld_dict dictionary.
	#####################################################################################################################

	print "Computing LD between all possible pairs of significant variants for each gene."
	logfile.write("Computing LD between all possible pairs of significant variants for each gene.\n")

	gene_ld_dict = {} 
	#key is geneID, values are unique IDs correspond to each possible pair of SNPs identified as significant for that gene.  
	#these values actually go to a nested dictionary, where the value is the r2 between those variants.

	for gene in snps_to_genes_dict.keys():

		ped_file = args.input_pedmap_directory + str(gene)
		ped_file_full = args.input_pedmap_directory + str(gene) + ".ped"

		if not os.path.isfile(ped_file_full):  #check that the file exists
			logfile.write("Cannot find file " + str(ped_file_full) + "\n")
			parser.error("Cannot find file " + str(ped_file_full))

		gene_ld_dict[gene] = {}

		for variant_a in snps_to_genes_dict[gene]:
			for variant_b in snps_to_genes_dict[gene]:
				ld_pair_id = variant_a + "_" + variant_b
				if variant_a == variant_b:
					gene_ld_dict[gene][ld_pair_id] = [1]
				
				else:
					ld_out = args.output_ld_directory + str(gene) + "_" + str(variant_a) + "_" + str(variant_b) +'_ld'
					ld_out_reverse = args.output_ld_directory + str(gene) + "_" + str(variant_a) + "_" + str(variant_b) +'_ld.log'
					ld_out_full = ld_out + '.log'

					if os.path.exists(ld_out_full):
						with open(ld_out_full, 'rU') as fin_inter:
							lines = filter(None, (line.strip() for line in fin_inter)) #gets rid of blank lines
							for line in lines:
								if (line.startswith("R-sq =")):
									rsquare = line.split(" ")[2]
									if float(rsquare) > float(args.r2_threshold):		#this is where we utilize the R2 threshold
										gene_ld_dict[gene][ld_pair_id] = [rsquare]
										ld_pair_id = variant_b + "_" + variant_a  #this ensures that the order of the variants doesn't matter.
										gene_ld_dict[gene][ld_pair_id] = [rsquare]
										print float(rsquare), float(args.r2_threshold)
					
					elif os.path.exists(ld_out_reverse):
						ld_out_full = ld_out_reverse
						with open(ld_out_full, 'rU') as fin_inter:
							lines = filter(None, (line.strip() for line in fin_inter)) #gets rid of blank lines
							for line in lines:
								if (line.startswith("R-sq =")):
									rsquare = line.split(" ")[2]
									if float(rsquare) > float(args.r2_threshold):		#this is where we utilize the R2 threshold
										gene_ld_dict[gene][ld_pair_id] = [rsquare]
										ld_pair_id = variant_b + "_" + variant_a  #this ensures that the order of the variants doesn't matter.
										gene_ld_dict[gene][ld_pair_id] = [rsquare]

					else:
						query = "plink --noweb --file " + ped_file + " --ld " + str(variant_a) + " " + str(variant_b) + " --out " + str(ld_out)
						p = subprocess.Popen(query.split(' '),stdout=subprocess.PIPE, stderr=subprocess.PIPE)
						stdout, stderr = p.communicate()
						logfile.write("Generating LD files with the following command:\n")
						logfile.write(query + "\n")
						logfile.write(stdout + "\n")
						logfile.write(stderr + "\n")

						if p.returncode:
						    raise Exception("Error occurred, see logfile.")
					
						ld_out_full = ld_out + '.log'

						if os.path.exists(ld_out_full):
							with open(ld_out_full, 'rU') as fin_inter:
								lines = filter(None, (line.strip() for line in fin_inter)) #gets rid of blank lines
								for line in lines:
									if (line.startswith("R-sq =")):
										rsquare = line.split(" ")[2]
										if float(rsquare) > float(args.r2_threshold):		#this is where we utilize the R2 threshold
											gene_ld_dict[gene][ld_pair_id] = [rsquare]
											ld_pair_id = variant_b + "_" + variant_a  #this ensures that the order of the variants doesn't matter.
											gene_ld_dict[gene][ld_pair_id] = [rsquare]

	print "DONE: Computing LD between all possible pairs of significant variants for each gene."
	logfile.write("DONE: Computing LD between all possible pairs of significant variants for each gene.\n")
					
	#####################################################################################################################
	#######   This section of code uses the dictionary built identify redundant interactions.
	#######   Specifically, for every significant interaction, we identify all pairs of variants that pass the 
	#######	  specified LD filters. These are stored in a new dictionary.
	#####################################################################################################################

	print gene_ld_dict

	print "Identifying redundant interactions."
	logfile.write("Identifying redundant interactions.\n")

	gene_redundant_pairs_dict = {}

	#we have to read the interaction file back in again.
	fin_signif_interactions = open(args.input_interactions_file, 'rU')
	int_reader = csv.reader(fin_signif_interactions, delimiter = '\t')

	for row in int_reader:
		gene_id = row[0]
		snp1    = row[3]
		snp2    = row[7]
		if gene_id not in snps_to_genes_dict:
			# Current partition does not contain this gene
			continue

		interaction_id_a = snp1 + "_" + snp2

		if gene_id not in gene_redundant_pairs_dict.keys():
			gene_redundant_pairs_dict[gene_id] = {}

		gene_redundant_pairs_dict[gene_id][interaction_id_a] = []

		fin_signif_interactions_two = open(args.input_interactions_file, 'rU')
		int_reader_two = csv.reader(fin_signif_interactions_two, delimiter = '\t')

		for row_two in int_reader_two:
			gene_id_two = row_two[0]
			snp1_two    = row_two[3]
			snp2_two    = row_two[7]

			interaction_id_b = snp1_two + "_" + snp2_two

			if gene_id == gene_id_two:

				#now check to see that it isn't itself
				if interaction_id_a != interaction_id_b:
					#now see if they pass the LD requirements. It is only in the LD dictionary if it passed.
					if (snp1+"_"+snp1_two) in gene_ld_dict[gene_id].keys():
						if (snp2+"_"+snp2_two) in gene_ld_dict[gene_id].keys():
							gene_redundant_pairs_dict[gene_id][interaction_id_a].append(interaction_id_b)

					if (snp1+"_"+snp2_two) in gene_ld_dict[gene_id].keys():
						if (snp2+"_"+snp1_two) in gene_ld_dict[gene_id].keys():
							gene_redundant_pairs_dict[gene_id][interaction_id_a].append(interaction_id_b)

		fin_signif_interactions_two.close()
	fin_signif_interactions.close()

	print "DONE: Identifying redudant interactions."
	logfile.write("DONE: Identifying redudant interactions.\n")

	return gene_redundant_pairs_dict

def cluster(gene_dict):
	
	print "Clustering redundant interactions."
	logfile.write("Clustering redundant interactions.\n")

	from collections import deque

	gene_ld_pairs = {}

	for gene in gene_dict:
		##Reassign global gdict to this particular gene snp-ld dict
		global gdict
		gdict = gene_dict[gene]
		candidates = deque(gdict.keys())
		clusters = []
		while candidates:
			p = candidates.popleft()
			c = ld_cluster(p)
			clusters.append(c)
			candidates = deque(set(candidates).difference(c))
		#gene-referenced cluster list
		gene_ld_pairs[gene] = clusters
	
	print "DONE: Clustering redundant interactions."
	logfile.write("DONE: Clustering redundant interactions.\n")

	return gene_ld_pairs


def ld_cluster(p):
  global gdict
  if p not in gdict: return []
  children = gdict[p]
  del gdict[p]
  if not children: 
    return [p]
  else:
    return [p]+[ldp for child in children for ldp in ld_cluster(child)]

def cluster_assignment(gene_ld_clusters):
	print "Generating output file containing clusters."
	logfile.write("Generating output file containing clusters.\n")

	fout_cluster = open("%s[%d].txt"%(args.output_results[:-4],args.ppidx), 'w')
	cluster_writer = csv.writer(fout_cluster, delimiter = "\t")

	fout_locus_name = "%s_locus_pairs[%d].txt"%(args.output_results[:-4],args.ppidx)
	print fout_locus_name
	fout_locus = open(fout_locus_name, 'w')
	locus_writer = csv.writer(fout_locus, delimiter = "\t")

	cluster_id = 0

	for gene in gene_ld_clusters.keys():

		for clusters in gene_ld_clusters[gene]:
			cluster_id = cluster_id + 1

			most_sig_pval = 1
			most_sig_pair = "NA"
			
			for snp_pair in clusters:

				fin_signif_interactions = open(args.input_interactions_file, 'rU')
				int_reader = csv.reader(fin_signif_interactions, delimiter = '\t')

				for row in int_reader:
					gene_id = row[0]
					snp1    = row[3]
					snp2    = row[7]
					disc_pval = float(row[10])

					if gene_id == gene:
						if snp1+"_"+snp2 == snp_pair:
							row.append(cluster_id)
							cluster_writer.writerow(row)

							print gene, cluster_id, snp1 + snp2

							if disc_pval < most_sig_pval:
								most_sig_pval = disc_pval
								most_sig_pair = row


				#print most_sig_pair


				fin_signif_interactions.close()

			locus_writer.writerow(most_sig_pair)

	fout_cluster.close()
	fout_locus.close()

if __name__ == '__main__':
	gene_dict = calc_between_pair_ld(snps_to_genes_dict)
	gene_ld_clusters = cluster(gene_dict)
	cluster_assignment(gene_ld_clusters)