"""
This script calculates the LD between variants that interact with one another.
"""

import csv
import argparse
import subprocess
import sys
import glob
import os



def calc_within_pair_ld():

	##################################################################################################
	parser = argparse.ArgumentParser(description = "This function calculates the r2 and d' between variants in the same interaction. If desired, an r2 and/or d' threshold can be specified." +
									" Interactions that do not pass this threshold will not be returned. By default, no interactions are removed.")

	parser.add_argument('-signif_int', '--input_interactions_file', help = "Specify the file containing significant interactions. This file is produced by the 'get_significant_interactions.py' script." +
						" This is required", 
						action='store', required = True)
	parser.add_argument('-ped_dir', '--input_pedmap_directory', help = "Specify the directory that contains the PED/MAP files used to run the INTERSNP analysis. This script assumes that the naming convention is dirctory_name/GeneID.ped." +
						" This is required", 
						action='store', required = True)
	parser.add_argument('-ld', '--ld_results', help = "Specify the name of the results file. This is required.", 
						action='store', required = True)
	parser.add_argument('-ld_dir', '--output_pedmap_directory', help = "Specify the directory in which to put the LD files that PLINK will generate. The final blackslash should be included (i.e., directory_name/)." +
						" This is required", action='store', required = True)
	parser.add_argument('-ind_file', '--individual_to_keep', help = "Provide a list of individuals to keep in the analysis. This should be in PLINK format." +
						" This is required", action='store', required = True)

	
	#Print error message if no input is provided.
	if len(sys.argv)==1:
		parser.print_help()
		sys.exit(1)
	args = parser.parse_args()
	
	#recreate the command given
	command = ""
	for com in sys.argv:
		command = command + str(com) + " "
	
	#create the logfile
	logfile_name = args.ld_results[:-4]+".log"
	logfile = open(logfile_name, 'w')
	logfile.write("You ran the command:\n")
	logfile.write(command + "\n")

	print "You ran the command:"
	print command
	
	#check input files can be found.
	input_files = [args.input_interactions_file, args.individual_to_keep]
	for input_file in input_files:
		if not os.path.isfile(input_file):
			logfile.write("Cannot find file " + str(input_file))
			parser.error("Cannot find file " + str(input_file))
	
	############################################################################################

	fin_pairs = open(args.input_interactions_file, 'rU')
	reader_pairs = csv.reader(fin_pairs, delimiter='\t')    
	
	fout_ld = open(args.ld_results, 'w')
	writer_ld = csv.writer(fout_ld, delimiter='\t')

	sig_interactions_count = 0
	sig_interactions_passing_thresholds_count = 0
	
	for row in reader_pairs:
		
		gene_id = row[0]
		snp1   = row[3]
		snp2   = row[7]

		genotype_file = args.input_pedmap_directory + str(gene_id)
		ld_out = args.output_pedmap_directory + str(gene_id) + "_" + str(snp1) + "_" + str(snp2) +'_ld'
		
		query = "plink --noweb --file " + genotype_file + " --keep " + str(args.individual_to_keep) + " --ld " + str(snp1) + " " + str(snp2) + " --out " + str(ld_out)

		p = subprocess.Popen(query.split(' '),stdout=subprocess.PIPE, stderr=subprocess.PIPE)
		stdout, stderr = p.communicate()
		logfile.write("Generating LD files with the following command:\n")
		logfile.write(query + "\n")
		logfile.write(stdout + "\n")
		logfile.write(stderr + "\n")

		if p.returncode:
		    raise Exception("Error occurred, see logfile.")
		
		ld_out_full = ld_out + '.log'
		
		sig_interactions_count = sig_interactions_count + 1

		if os.path.exists(ld_out_full):
			with open(ld_out_full, 'rU') as fin_inter:
				lines = filter(None, (line.strip() for line in fin_inter)) #gets rid of blank lines
							   
				for line in lines:

					if (line.startswith("R-sq =")):
						rsquare = line.split(" ")[2]
						dprime = line.split(" ")[9]
						results = [gene_id, snp1, snp2, rsquare, dprime]
						writer_ld.writerow(results)
						sig_interactions_passing_thresholds_count = sig_interactions_passing_thresholds_count + 1

	print "There were " + str(sig_interactions_count) + " interactions tested for LD."

	logfile.write("There were " + str(sig_interactions_count) + " interactions tested for LD.\n")

	fout_ld.close()
	fin_pairs.close()
	logfile.close()



if __name__ == '__main__':
	calc_within_pair_ld()