"""
This script identifies the cis-eQTL that most mediates the significance of the interactions. It also checks to see if that variant is within a specified LD level with either of the interacting variants.
If so, that cis-eQTL is not considered.
"""

import csv
import argparse
import subprocess
import sys
import glob
import os

##################################################################################################
parser = argparse.ArgumentParser(description = "This script identifies the cis-eQTL that most mediates the significance of the interactions. It also checks to see if that variant is within a specified LD level with either of the " +
			" interacting variants. If so, that cis-eQTL is not considered.")

parser.add_argument('-pval', '--pvalue_threshold', help = "Specify the p-value threshold used to identify cis-eQTL. This is required to know which INTERSNP results to parse.", 
					action='store', required = True)
parser.add_argument('-signif_int', '--input_interactions_file', help = "Specify the path to the significant interactions output file from the get_significant_interactions.py script. This is required.", 
						action='store', required = True)
parser.add_argument('-intersnp_dir', '--intersnp_directory', help = "Specify the directory containing the INTERSNP files generated by conditional_cis_eqtl_intersnp.py. Note: The final blackslash should be included (i.e., directory_name/)." +
					" This is required.", action='store', required = True)
parser.add_argument('-ped_dir', '--input_pedmap_directory', help = "Specify the directory that contains the PED/MAP files used to run the INTERSNP analysis. Note: The final blackslash should be included (i.e., directory_name/)" + 
					"This script assumes that the naming convention is dirctory_name/GeneID.ped. It will test all ped/map files in this directory." +
					" This is required.", action='store', required = True)
parser.add_argument('-rsquare', '--rsquare_threshold', type=float, help = "Specify the path to the significant interactions output file from the get_significant_interactions.py script. This is required.", 
						action='store', required = True)
parser.add_argument('-results_file', '--results_file_name', help = "Specify the name for the results of the conditional cis-eQTL analysis. This is required.", 
						action='store', required = True)

#Print error message if no input is provided.
if len(sys.argv)==1:
	parser.print_help()
	sys.exit(1)
args = parser.parse_args()

#recreate the command given
command = ""
for com in sys.argv:
	command = command + str(com) + " "

#create the logfile
logfile_name = args.results_file_name+".log"
logfile = open(logfile_name, 'w')
logfile.write("You ran the command:\n")
logfile.write(command + "\n")

print "You ran the command:"
print command

#check input files can be found.
input_files = [args.input_interactions_file]
for input_file in input_files:
	if not os.path.isfile(input_file):
		logfile.write("Cannot find file " + str(input_file))
		parser.error("Cannot find file " + str(input_file))

############################################################################################

def get_ld(gene_id,snp1,snp2):

	genotype_file = args.input_pedmap_directory + str(gene_id) + '_maf05_filtered_pheno'
	ld_out = args.intersnp_directory + str(gene_id) + "_" + str(snp1) + "_" + str(snp2) +'_ld'
	ld_out_full = ld_out + '.log'
	if os.path.exists(ld_out_full):
		with open(ld_out_full, 'rU') as fin_inter:
			lines = filter(None, (line.strip() for line in fin_inter)) #gets rid of blank lines		   
			for line in lines:
				if (line.startswith("R-sq =")):
					rsquare = line.split(" ")[2]
					return float(rsquare)

	else:
		query = "plink --noweb --file " + genotype_file + " --ld " + str(snp1) + " " + str(snp2) + " --out " + str(ld_out)
		p = subprocess.Popen(query.split(' '),stdout=subprocess.PIPE, stderr=subprocess.PIPE)
		stdout, stderr = p.communicate()
		logfile.write("Generating LD files with the following command:\n")
		logfile.write(query + "\n")
		logfile.write(stdout + "\n")
		logfile.write(stderr + "\n")
		
		if p.returncode:
			raise Exception("Error occurred, see logfile.")

		if os.path.exists(ld_out_full):
			with open(ld_out_full, 'rU') as fin_inter:
				lines = filter(None, (line.strip() for line in fin_inter)) #gets rid of blank lines		   
				for line in lines:
					if (line.startswith("R-sq =")):
						rsquare = line.split(" ")[2]
						return float(rsquare)

def conditional_results():

	
	fout = open(args.results_file_name, 'w')
	results_writer = csv.writer(fout, delimiter="\t")

	# 1. Read all interacting pairs
	fin_int = open(args.input_interactions_file)
	int_reader = csv.reader(fin_int, delimiter="\t")


	count_int = 0
	# 2. For each interacting pair of SNPs
	for interaction in int_reader:
		gene_id = interaction[0]
		rsid_a = interaction[3]
		rsid_b = interaction[7]

		count_int = count_int + 1

		cond_pvals = []
		# 3. Get the conditional p-value for each cis-eqtl
		intersnp_file_name = args.intersnp_directory + gene_id + "_" + rsid_a + "_" + rsid_b + '_conditional_BestMarkerCombi3.txt'
		if not os.path.isfile(intersnp_file_name):
			logfile.write("Cannot find file " + str(intersnp_file))
			results = [gene_id,rsid_a,rsid_b,"NA","NA"]
			results_writer.writerow(results)
			print results, count_int
			continue # continue to the next interacting pair
		
		else:
			with open(intersnp_file_name, 'rU') as intersnp_file:
				intersnp_file.readline()
				lines = filter(None, (line.rstrip() for line in intersnp_file))
				for line in lines:
					if (line.startswith(" Note:") or line.startswith(' e.g.:')):
						continue
					else:
						row = line.split("\t")
						# Read all cis-eqtls and their conditional p-values
						if row[6] == rsid_a:
							if row[10] == rsid_b:
								cond_pvals.append((float(row[13]),row[2]))
		
		# 4. Sort the cis-eqtls by decreasing conditional p-value
		cond_pvals  = sorted(cond_pvals,reverse=True)
		# 5. Identify the cis-eqtl with the highest p-value that isn't in perfect LD with either interacting SNP
		for cond_pval,cis_eqtl in cond_pvals:
			ldA = get_ld(gene_id,cis_eqtl,rsid_a)
			ldB = get_ld(gene_id,cis_eqtl,rsid_b)
			# 6. Exit the loop only when the cis_eqtl is out of LD with the pair
			if not (ldA >= args.rsquare_threshold or ldB >= args.rsquare_threshold):
				max_pval = cond_pval
				max_pval_cis_eqtl = cis_eqtl
				# 7. Print results.
				results = [gene_id,rsid_a,rsid_b,max_pval_cis_eqtl,max_pval]
				results_writer.writerow(results)

				print results, count_int

				break # We've found the solution, stop the loop
	
			

	fin_int.close()
	fout.close()


if __name__ == '__main__':
	conditional_results()