"""
This script generates the selection file for the conditional cis-eQTL analysis, and then runs it.
"""

import csv
import argparse
import subprocess
import sys
import glob
import os

##################################################################################################
parser = argparse.ArgumentParser(description = "This script takes a list of significant cis-eQTL for each gene, and creates each combination of significant interactions and nominal cis-eQTL."
	+ "This script creates an output file for each gene in the same directory as the INTERSNP results.")

parser.add_argument('-ped_dir', '--input_pedmap_directory', help = "Specify the directory that contains the PED/MAP files used to run the INTERSNP analysis. Note: The final blackslash should be included (i.e., directory_name/)" + 
					"This script assumes that the naming convention is dirctory_name/GeneID.ped. It will test all ped/map files in this directory." +
					" This is required.", action='store', required = True)
parser.add_argument('-out_dir', '--output_directory', help = "Specify the directory in which to put the INTERSNP files. Note: The final blackslash should be included (i.e., directory_name/)." +
					" This is required.", action='store', required = True)
parser.add_argument('-combi_dir', '--input_combination_directory', help = "Specify the directory containing the combinations to be tested. Note: The final blackslash should be included (i.e., directory_name/)." +
					" It assumes that you have used the naming conventions of the generate_combinations_for_conditional_analysis.py script. This is required.", action='store', required = True)
parser.add_argument('-covariate_file', '--covariate_file', help = "Specify the covariate file name. Check the format for this file in the INTERSNP manual.", action='store', required = False)
parser.add_argument('-covariates', '--covariates', help = "Specify the covariates to include, using the syntax specified in the INTERSNP manual.", action='store', required = False)
parser.add_argument('-model', '--model_file', help = "Provide a model_file, using the syntax specified in the INTERSNP manual.", action='store', required = True)
parser.add_argument('-pval', '--pvalue_threshold', help = "Specify the p-value threshold for significance (ex.: 0.05). This is required to know which cis-eQTL to test.", 
					action='store', required = True)
parser.add_argument('-int', '--input_interactions_file', help = "Specify the path to the significant interactions output file from the get_significant_interactions.py script. This is required.", 
						action='store', required = True)
parser.add_argument('--ppart',type=int,default=1,help="Number of partition")
parser.add_argument('--ppidx',type=int,default=0,help='Index of partition to process')
args = parser.parse_args()

#recreate the command given
command = ""
for com in sys.argv:
	command = command + str(com) + " "

#create the logfile
logfile_name = args.output_directory+"conditional_analysis.log"
logfile = open(logfile_name, 'w')
logfile.write("You ran the command:\n")
logfile.write(command + "\n")

print "You ran the command:"
print command

#check input files can be found.
input_files = [args.model_file, args.covariate_file]
for input_file in input_files:
	if not os.path.isfile(input_file):
		logfile.write("Cannot find file " + str(input_file))
		parser.error("Cannot find file " + str(input_file))

############################################################################################

def conditional_analysis():

	interactions=[]
	
	fin_int = open(args.input_interactions_file)
	int_reader = csv.reader(fin_int, delimiter="\t")

	for interaction in int_reader:
		gene_id = interaction[0]
		rsid_a  = interaction[3]
		rsid_b  = interaction[7]

		interaction_id = gene_id + rsid_a + rsid_b
		interactions.append(interaction_id)
	fin_int.close()

	for i,inter in enumerate(interactions):
		if i%args.ppart != args.ppidx: continue

		fin_int = open(args.input_interactions_file)
		int_reader = csv.reader(fin_int, delimiter="\t")

		for interaction in int_reader:
			gene = interaction[0]
			rsid_a  = interaction[3]
			rsid_b  = interaction[7]

			if inter == gene+rsid_a+rsid_b:

				fout_name_selection = args.output_directory + gene +"_" + rsid_a + "_" + rsid_b + "_conditional_trio_selection.txt"
				#print fout_name_selection
				fout_sel = open(fout_name_selection, 'w')
				
				map_file = args.input_pedmap_directory + gene +'_maf05_filtered_pheno.map'
				map_info = 'MAP    ' + map_file
				
				ped_file = args.input_pedmap_directory + gene +'_maf05_filtered_pheno.ped'
				ped_info = 'PED    ' + ped_file
				
				intersnp_output = args.output_directory + gene +"_" + rsid_a + "_" + rsid_b + '_conditional_'
				intersnp_output_info = 'OUTPUTNAME    ' + intersnp_output
				
				covariate_file = args.covariate_file

				covariate_info = 'COVARIATEFILE    ' + covariate_file
				
				model_file = args.model_file
				model_info = 'MODELFILE    ' + model_file
				
				combi_file = args.input_combination_directory + gene +"_" + rsid_a + "_" + rsid_b + '_maf05_filtered_pheno_cis_eqtl_' + str(args.pvalue_threshold) + '_combinations.txt'
				combi_info = 'COMBIFILE    ' + combi_file
				
				fout_sel.write(map_info + '\n')
				fout_sel.write(ped_info + '\n')
				fout_sel.write(covariate_info + '\n')
				fout_sel.write(intersnp_output_info + '\n')
				fout_sel.write(model_info + "\n")
				fout_sel.write(combi_info + "\n")
				fout_sel.write("COVARIATES " + str(args.covariates) + "\n")
				fout_sel.write('QT    1    // Quantitative Trait? 1=yes, 0=no\nCOMBILIST    1    //use combilist provided\nTHREE_MARKER 1 // perform the conditional_cis_eqlt analysis\nTWO_MARKER    0    // Two-marker-analysis: 1=yes, 0=no\nPRETEST    0    // Pretest: 1=yes, 0=no\nTEST    M    // Test: 6 = GENOTYPIC INTERACTION, with both additive and dominant components included\nSINGLE_MARKER    2    // 2 = genotype test w. 2 df\n PRINTTOP 500000\nEND')
				fout_sel.close()

				#okay, now that I wrote the selection file, let's actually run INTERSNP!
				query = "INTERSNP_1.15/intersnp " + fout_name_selection

				p = subprocess.Popen(query.split(' '),stdout=subprocess.PIPE, stderr=subprocess.PIPE)
				stdout, stderr = p.communicate()
				logfile.write(query + "\n")
				logfile.write(stdout + "\n")
				logfile.write(stderr + "\n")
	fin_int.close()



if __name__ == '__main__':
	conditional_analysis()