"""
This script generates the INTERSNP selection file, then runs the interaction analysis.
"""

import csv
import argparse
import subprocess
import sys
import glob
import os

def intersnp_analysis():
	
	##################################################################################################
	parser = argparse.ArgumentParser(description = "This function generates selection files for INTERSNP, and then uses those selection files to run the interaction analysis.")


	parser.add_argument('-i', '--input_pedmap_directory', help = "Specify the directory that contains the PED/MAP files used to run the INTERSNP analysis. Note: The final blackslash should be included (i.e., directory_name/)" + 
						"This script assumes that the naming convention is dirctory_name/GeneID.ped." +
						" This is required. All PED/MAP files in this directory will be tested.", action='store', required = True)
	parser.add_argument('-o', '--output_directory', help = "Specify the directory in which to put the INTERSNP files. Note: The final blackslash should be included (i.e., directory_name/)." +
						" This is required.", action='store', required = True)
	parser.add_argument('-covariate_file', '--covariate_file', help = "Specify the covariate file name. Check the format for this file in the INTERSNP manual.", action='store', required = False)
	parser.add_argument('-covariates', '--covariates', help = "Specify the covariates to include, using the syntax specified in the INTERSNP manual.", action='store', required = False)

	#Print error message if no input is provided.
	if len(sys.argv)==1:
		parser.print_help()
		sys.exit(1)
	args = parser.parse_args()
	
	#recreate the command given
	command = ""
	for com in sys.argv:
		command = command + str(com) + " "
	
	#create the logfile
	logfile_name = args.output_directory+"intersnp_analysis.log"
	logfile = open(logfile_name, 'w')
	logfile.write("You ran the command:\n")
	logfile.write(command + "\n")

	print "You ran the command:"
	print command
	
	#check input files can be found.
	input_files = [args.covariate_file]
	for input_file in input_files:
		if not os.path.isfile(input_file):
			logfile.write("Cannot find file " + str(input_file))
			parser.error("Cannot find file " + str(input_file))
	
	############################################################################################

	genes = glob.glob(args.input_pedmap_directory + '*.ped')
	
	for gene in genes:
		
		gene = gene.split("/")
		gene = gene[len(gene)-1]
		gene = gene[:-4]
		
		fout_name_selection = args.output_directory + gene + '_selection.txt'
		fout_sel = open(fout_name_selection, 'w')
		
		map_file = args.input_pedmap_directory + gene + '.map'
		map_info = 'MAP    ' + map_file
		
		ped_file = args.input_pedmap_directory + gene + '.ped'
		ped_info = 'PED    ' + ped_file
		
		intersnp_output = args.output_directory + gene
		intersnp_output_info = 'OUTPUTNAME    ' + intersnp_output

		covariate_info = 'COVARIATEFILE    ' + args.covariate_file
		
		fout_sel.write(map_info + '\n')
		fout_sel.write(ped_info + '\n')
		fout_sel.write(covariate_info + '\n')
		fout_sel.write(intersnp_output_info + '\n')
		fout_sel.write( 'QT    1    // Quantitative Trait? 1=yes, 0=no\nTWO_MARKER    1    // Two-marker-analysis: 1=yes, 0=no\nPRETEST    0    // Pretest: 1=yes, 0=no\nTEST    6    // Test: 6 = GENOTYPIC INTERACTION, with both additive and dominant components included\nSINGLE_MARKER    2    // 2 = genotype test w. 2 df\n PRINTTOP 500000\nCOVARIATES ' + str(args.covariates) + '\nEND')
		fout_sel.close()

		query = "intersnp " + fout_name_selection

		p = subprocess.Popen(query.split(' '),stdout=subprocess.PIPE, stderr=subprocess.PIPE)
		stdout, stderr = p.communicate()
		logfile.write(query + "\n")
		logfile.write(stdout + "\n")
		logfile.write(stderr + "\n")

	logfile.close()

if __name__ == '__main__':
	intersnp_analysis()