"""
Genotype check.
This script:

A. Takes and input file of the significant results.
B. Gets a ped/map file for each of them.
C. Looks at the 0/1/2 column for each variant in the ped/map file.
D. Checks to see if all 0/1/2 + 0/1/2 combinations are present.
E. Provides an output file 
"""

import csv
import argparse
import subprocess
import sys
import os


#this generates the genotype results
def complete_genotypes():

	##################################################################################################
	parser = argparse.ArgumentParser(description = "This function checks whether or not there are missing genotype classes for significant interactions.")
	parser.add_argument('-i', '--input_interactions_file', help = "Specify the path to the significant interactions output file from the get_significant_interactions.py script. This is required.", 
						action='store', required = True)
	parser.add_argument('-p', '--input_pedmap_directory', help = "Specify the directory that contains the PED/MAP files used to run the INTERSNP analysis. This script assumes that the naming convention is dirctory_name/GeneID.ped." +
						" This is required", 
						action='store', required = True)
	parser.add_argument('-o', '--output_directory', help = "Specify the directory for output raw files from PLINK. One file will be generated per interaction. This is required", 
						action='store', required = True)
	parser.add_argument('-r', '--output_results', help = "Specify the the name of the results file. This is required.", 
						action='store', required = True)
	
	#Print error message if no input is provided.
	if len(sys.argv)==1:
		parser.print_help()
		sys.exit(1)
	args = parser.parse_args()
	
	#recreate the command given
	command = ""
	for com in sys.argv:
		command = command + str(com) + " "
	
	#create the logfile
	logfile_name = args.output_results[:-4]+".log"
	logfile = open(logfile_name, 'w')
	logfile.write("You ran the command:\n")
	logfile.write(command + "\n")

	print "You ran the command:"
	print command
	
	#check input files can be found.
	input_files = [args.input_interactions_file]
	for input_file in input_files:
		if not os.path.isfile(input_file):
			logfile.write("Cannot find file " + str(input_file))
			parser.error("Cannot find file " + str(input_file))
	
	############################################################################################

	fin_int = open(args.input_interactions_file, 'rU')
	int_reader = csv.reader(fin_int, delimiter = "\t")

	fout_results = open(args.output_results, 'w')
	results_writer = csv.writer(fout_results, delimiter="\t")

	header = ["Gene_ID","SNP_A","SNP_B","00","01","02","10","11","12","20","21","22"]
	results_writer.writerow(header)

	total_interactions = 0
	interactions_with_missing = 0

	for row in int_reader:

		total_interactions = total_interactions + 1

		gene 	= row[0]
		rsid_a  = row[3]
		rsid_b  = row[7]

		recode_name = args.output_directory + str(gene) + "_" + rsid_a + "_" + rsid_b

		#Generate the plink file we need
		plink_file =  args.input_pedmap_directory + str(gene) 
		query = "plink --noweb --file " + plink_file + " --snps " + str(rsid_a) + "," + str(rsid_b) + " --recodeAD --out " + recode_name

		p = subprocess.Popen(query.split(' '),stdout=subprocess.PIPE, stderr=subprocess.PIPE)
		stdout, stderr = p.communicate()
		logfile.write("Generating new PED/MAP files with the following command:\n")
		logfile.write(query + "\n")
		logfile.write(stdout + "\n")
		logfile.write(stderr + "\n")

		if p.returncode:
		    raise Exception("Error occurred, see logfile.")

		#read the ped/map file in
		fin_ped_name = recode_name + ".raw"
		fin_ped = open(fin_ped_name, 'rU')
		fin_ped.readline()
		reader_ped = csv.reader(fin_ped, delimiter=" ")

		genotypes_observed = []

		for individual_in_ped in reader_ped:
			genotype_a = individual_in_ped[6]
			genotype_b = individual_in_ped[8]
			genotype_combo = str(genotype_a) + str(genotype_b)
			genotypes_observed.append(genotype_combo)
		fin_ped.close()

		genotypes_observed = set(genotypes_observed)
		all_geno = ["00","01","02","10","11","12","20","21","22"]

		results = [gene, rsid_a, rsid_b]

		for geno in all_geno:
			status = "NA"
			if geno not in genotypes_observed:
				status = "Missing"
			if geno in genotypes_observed:
				status = "Present"
			results.append(status)

		results_writer.writerow(results)

		if 'Missing' in results:
			interactions_with_missing = interactions_with_missing + 1

	print "Of the " + str(total_interactions) + "  total interactions, " + str(interactions_with_missing) + " have a missing genotype combination." 
	logfile.write("Of the " + str(total_interactions) + "  total interactions, " + str(interactions_with_missing) + " have a missing genotype combination.\n")

	logfile.close()
	fin_int.close()
	fout_results.close()


if __name__ == '__main__':
	complete_genotypes()
