'''
This script performs a single-marker cis-eQTL analysis on specified ped/map files.
'''

import csv
import argparse
import subprocess
import sys
import glob
import os


##################################################################################################
parser = argparse.ArgumentParser(description = "This script uses INTERSNP to identify cis-eQTL from ped/map files. Any single-marker model file can be specified.")

parser.add_argument('-i', '--input_pedmap_directory', help = "Specify the directory that contains the PED/MAP files used to run the INTERSNP analysis. Note: The final blackslash should be included (i.e., directory_name/)" + 
					"This script assumes that the naming convention is dirctory_name/GeneID.ped. It will test all ped/map files in this directory." +
					" This is required. All PED/MAP files in this directory will be tested.", action='store', required = True)
parser.add_argument('-o', '--output_directory', help = "Specify the directory in which to put the INTERSNP files. Note: The final blackslash should be included (i.e., directory_name/)." +
					" This is required.", action='store', required = True)
parser.add_argument('-covariate_file', '--covariate_file', help = "Specify the covariate file name. Check the format for this file in the INTERSNP manual.", action='store', required = False)
parser.add_argument('-covariates', '--covariates', help = "Specify the covariates to include, using the syntax specified in the INTERSNP manual.", action='store', required = False)
parser.add_argument('-m', '--model_file', help = "Provide a model_file, using the syntax specified in the INTERSNP manual.", action='store', required = True)
parser.add_argument('--ppart',type=int,default=1,help="Number of partition")
parser.add_argument('--ppidx',type=int,default=0,help='Index of partition to process')
args = parser.parse_args()

#recreate the command given
command = ""
for com in sys.argv:
	command = command + str(com) + " "

#create the logfile
logfile_name = "%s/intersnp_analysis[%d].log"%(args.output_directory.rstrip('/'),args.ppidx)
logfile = open(logfile_name, 'w')
logfile.write("You ran the command:\n")
logfile.write(command + "\n")

print "You ran the command:"
print command

#check input files can be found.
input_files = [args.covariate_file]
for input_file in input_files:
	if not os.path.isfile(input_file):
		logfile.write("Cannot find file " + str(input_file))
		parser.error("Cannot find file " + str(input_file))

# Identify genes from ped file
fdir = args.input_pedmap_directory
genes = sorted(glob.glob(fdir+'*.ped'))

# Identify which genes are assigned to this partition
print "\nIdentifying genes in partition %d of %d for subset processing..."%(args.ppidx,args.ppart)
logfile.write("Identifying genes in partition %d of %d for subset processing..."%(args.ppidx,args.ppart))
genes = [g for i,g in enumerate(genes) if i%args.ppart == args.ppidx]
print "Partition has been populated (%d genes) and is ready for parallel processing.\n"%(len(genes))
logfile.write("Partition has been populated and is ready for parallel processing.")

############################################################################################


def selection_files(genes):	
	
	count = 0
	for gene in genes:
		
		count = count +1
		print count

		gene = gene.split("/")
		gene = gene[len(gene)-1]
		gene = gene[:-4]
		
		fout_name_selection = args.output_directory + gene + "_selection.txt"
		fout_sel = open(fout_name_selection, 'w')
		
		map_file = fdir + gene + '.map'
		map_info = 'MAP    ' + map_file
		
		ped_file = fdir + gene + '.ped'
		ped_info = 'PED    ' + ped_file
		
		intersnp_output = args.output_directory + gene + "_"
		intersnp_output_info = 'OUTPUTNAME    ' + intersnp_output
		
		covariate_file = args.covariate_file
		covariate_info = 'COVARIATEFILE    ' + covariate_file
		
		model_file = args.model_file
		model_info = 'MODELFILE    ' + model_file
		
		fout_sel.write(map_info + '\n')
		fout_sel.write(ped_info + '\n')
		fout_sel.write(covariate_info + '\n')
		fout_sel.write(intersnp_output_info + '\n')
		fout_sel.write(model_info + "\n")
		fout_sel.write("COVARIATES " + str(args.covariates) + "\n")
		fout_sel.write( 'QT    1    // Quantitative Trait? 1=yes, 0=no\nTWO_MARKER    0    // Two-marker-analysis: 1=yes, 0=no\nPRETEST    0    // Pretest: 1=yes, 0=no\nTEST    M    // Test: 6 = GENOTYPIC INTERACTION, with both additive and dominant components included\nSINGLE_MARKER    2    // 2 = genotype test w. 2 df\n PRINTTOP 500000\nEND')
		fout_sel.close()

		query = "INTERSNP_1.15/intersnp " + fout_name_selection

		p = subprocess.Popen(query.split(' '),stdout=subprocess.PIPE, stderr=subprocess.PIPE)
		stdout, stderr = p.communicate()
		logfile.write(query + "\n")
		logfile.write(stdout + "\n")
		logfile.write(stderr + "\n")



if __name__ == '__main__':
	selection_files(genes)
