"""
This script generates the INTERSNP selection file, then runs the interaction analysis.
"""

import csv
import argparse
import subprocess
import sys
import glob
import os

def intersnp_analysis():
	
	##################################################################################################
	parser = argparse.ArgumentParser(description = "This function generates selection files for INTERSNP, and then uses those selection files to run the interaction analysis. Only interactions identified as significant in the" + 
		"Discovery analysis are tested.")

	parser.add_argument('-ped_dir', '--input_pedmap_directory', help = "Specify the directory that contains the PED/MAP files used to run the INTERSNP analysis. Note: The final blackslash should be included (i.e., directory_name/)" + 
						"This script assumes that the naming convention is dirctory_name/GeneID.ped." +
						" This is required. All PED/MAP files in this directory will be tested.", action='store', required = True)
	parser.add_argument('-signif_int', '--input_interactions_file', help = "Specify the file containing significant interactions. This file is produced by the 'get_significant_interactions.py' script." +
						" This is required", 
						action='store', required = True)
	parser.add_argument('-o', '--output_directory', help = "Specify the directory in which to put the INTERSNP files. Note: The final blackslash should be included (i.e., directory_name/)." +
						" This is required.", action='store', required = True)
	parser.add_argument('-covariate_file', '--covariate_file', help = "Specify the covariate file name. Check the format for this file in the INTERSNP manual.", action='store', required = False)
	parser.add_argument('-covariates', '--covariates', help = "Specify the covariates to include, using the syntax specified in the INTERSNP manual.", action='store', required = False)

	#Print error message if no input is provided.
	if len(sys.argv)==1:
		parser.print_help()
		sys.exit(1)
	args = parser.parse_args()
	
	#recreate the command given
	command = ""
	for com in sys.argv:
		command = command + str(com) + " "
	
	#create the logfile
	logfile_name = args.output_directory+"intersnp_analysis.log"
	logfile = open(logfile_name, 'w')
	logfile.write("You ran the command:\n")
	logfile.write(command + "\n")

	print "You ran the command:"
	print command
	
	#check input files can be found.
	input_files = [args.covariate_file, args.input_interactions_file]
	for input_file in input_files:
		if not os.path.isfile(input_file):
			logfile.write("Cannot find file " + str(input_file))
			parser.error("Cannot find file " + str(input_file))
	
	############################################################################################

	print "Identifying the interactions to be tested."

	significant_genes = []

	fin_int = open(args.input_interactions_file, 'rU')
	signif_int = csv.reader(fin_int, delimiter = "\t")

	for row in signif_int:
		gene_id = row[0]
		if row[33] == "Rare variants only.":
			significant_genes.append(gene_id)
	fin_int.close()

	significant_genes = set(significant_genes)
	significant_genes = list(significant_genes)

	for gene in significant_genes:

		fout_combi_name = args.output_directory + gene + "_combinations.txt"
		fout_combi = open(fout_combi_name, 'w')
		combi_writer = csv.writer(fout_combi, delimiter="\t")

		fin_int = open(args.input_interactions_file, 'rU')
		signif_int = csv.reader(fin_int, delimiter = "\t")
		for row in signif_int:
			gene_id = row[0]
			
			if gene_id == gene:
				snpa = row[3]
				snpb = row[7]

				combi = [snpa, snpb]
				combi_writer.writerow(combi)
		fin_int.close()
		fout_combi.close()

	
	for gene in significant_genes:
		
		fout_name_selection = args.output_directory + gene + '_selection.txt'
		fout_sel = open(fout_name_selection, 'w')
		
		map_file = args.input_pedmap_directory + gene + '_maf05_filtered_pheno_probe_filtered.map'
		map_info = 'MAP    ' + map_file
		
		ped_file = args.input_pedmap_directory + gene + '_maf05_filtered_pheno_probe_filtered.ped'
		ped_info = 'PED    ' + ped_file
		
		intersnp_output = args.output_directory + gene
		intersnp_output_info = 'OUTPUTNAME    ' + intersnp_output

		combi_info = 'COMBIFILE    ' + args.output_directory + gene + "_combinations.txt"

		covariate_info = 'COVARIATEFILE    ' + args.covariate_file
		
		fout_sel.write(map_info + '\n')
		fout_sel.write(ped_info + '\n')
		fout_sel.write(covariate_info + '\n')
		fout_sel.write(intersnp_output_info + '\n')
		fout_sel.write(combi_info + "\n")
		fout_sel.write( 'COMBILIST	1	//test only the specified combinations\nQT    1    // Quantitative Trait? 1=yes, 0=no\nTWO_MARKER    1    // Two-marker-analysis: 1=yes, 0=no\nPRETEST    0    // Pretest: 1=yes, 0=no\nTEST    6    // Test: 6 = GENOTYPIC INTERACTION, with both additive and dominant components included\nSINGLE_MARKER    2    // 2 = genotype test w. 2 df\n PRINTTOP 500000\nCOVARIATES ' + str(args.covariates) + '\nEND')
		fout_sel.close()

		query = "INTERSNP_1.15/intersnp " + fout_name_selection

		p = subprocess.Popen(query.split(' '),stdout=subprocess.PIPE, stderr=subprocess.PIPE)
		stdout, stderr = p.communicate()
		logfile.write(query + "\n")
		logfile.write(stdout + "\n")
		logfile.write(stderr + "\n")

	logfile.close()

if __name__ == '__main__':
	intersnp_analysis()