"""
This script identifies all variants within the specified genomic coordinates for probe ids.
"""

import csv
import argparse
import subprocess
import sys
import os

def calc_frequency():

	##################################################################################################
	parser = argparse.ArgumentParser(description = "This function determines the presence / frequency of variants in probe binding sites.")
	parser.add_argument('-p', '--input_probe_ids', help = "Provide a tab delimited list of probe_ids. The probe_id needs to be in the first column, and the gene_id in the 12 column."
						+ " Other columns will be ignored. A header is assumed. This is required.", 
						action='store', required = True)
	parser.add_argument('-i', '--input_interactions_file', help = "Specify the path to the significant interactions output file from the get_significant_interactions.py script. This is required.", 
						action='store', required = True)
	parser.add_argument('-v', '--input_vcf_directory', help = "Specify the directory that contains the vcf files used to run the INTERSNP analysis. This script assumes that the naming convention is dirctory_name/probeID.ped." +
						" This is required", 
						action='store', required = True)
	parser.add_argument('-o', '--output_directory', help = "Specify the directory for output frequency files. One file will be generated per probe. This is required", 
						action='store', required = True)
	parser.add_argument('-r', '--output_results', help = "Specify the the name of the results file. This is required.", 
						action='store', required = True)
	
	#Print error message if no input is provided.
	if len(sys.argv)==1:
		parser.print_help()
		sys.exit(1)
	args = parser.parse_args()
	
	#recreate the command given
	command = ""
	for com in sys.argv:
		command = command + str(com) + " "
	
	#create the logfile
	logfile_name = args.output_results[:-4]+".log"
	logfile = open(logfile_name, 'w')
	logfile.write("You ran the command:\n")
	logfile.write(command + "\n")

	print "You ran the command:"
	print command
	
	#check input files can be found.
	input_files = [args.input_probe_ids, args.input_interactions_file]
	for input_file in input_files:
		if not os.path.isfile(input_file):
			logfile.write("Cannot find file " + str(input_file))
			parser.error("Cannot find file " + str(input_file))
	
	############################################################################################

	genes_without_variants = []
	genes_with_common_variants = []
	genes_without_common_variants = []

	fin_probes = open(args.input_probe_ids, 'rU')
	fin_probes.readline() #header
	fin_reader = csv.reader(fin_probes, delimiter="\t")
	
	for probe in fin_reader:
		probe_id = probe[0]
		gene_id = probe[11]
		
		vcf_prefix = args.output_directory + probe_id
		vcf_file = args.input_vcf_directory + probe_id + ".recode.vcf"

		if os.path.isfile(vcf_file):
		
			query = "vcftools --vcf " + vcf_file + " --freq --out " + vcf_prefix

			p = subprocess.Popen(query.split(' '),stdout=subprocess.PIPE, stderr=subprocess.PIPE)
			stdout, stderr = p.communicate()
			logfile.write("Generating freq files with the following command:\n")
			logfile.write(query + "\n")
			logfile.write(stdout + "\n")
			logfile.write(stderr + "\n")

			if p.returncode:
				raise Exception("Error occurred, see logfile.")

			vcf_frq_file = args.output_directory + probe_id + '.frq'
			fin_frq = open(vcf_frq_file, 'rU')
			fin_frq.readline()
			frq_reader = csv.reader(fin_frq, delimiter = "\t")
		
			variants = []
			common_variants = []
			rare_variants = []
		
			for variant in frq_reader:
				variant_id = str(variant[0]) + ":" + str(variant[1])
				allele_a = variant[4]
				allele_a = allele_a.split(":")
				allele_a_freq = allele_a[1]
	
				if str(allele_a_freq) != "1":
					variants.append(variant_id)
					if float(allele_a_freq) > 0.95:
						rare_variants.append(variant_id)
					else:
						common_variants.append(variant_id)
			fin_frq.close()
		
			if len(variants) == 0:
				genes_without_variants.append(gene_id)
			
			elif len(common_variants) == 0:
				genes_without_common_variants.append(gene_id)
				
			else:
				genes_with_common_variants.append(gene_id)
		else:
			logfile.write("Cannot find file " + vcf_file)
			parser.error("Cannot find file " + vcf_file)

	fin_probes.close()

	print "There are " + str(len(genes_without_variants)) + " probes without any variants."
	logfile.write("There are " + str(len(genes_without_variants)) + " probes without any variants.\n")
	print "There are " + str(len(genes_without_common_variants)) + " probes with only rare variants."
	logfile.write("There are " + str(len(genes_without_common_variants)) + " probes with only rare variants.\n")
	print "There are " + str(len(genes_with_common_variants)) + " probes with a common variant."
	logfile.write("There are " + str(len(genes_with_common_variants)) + " probes with a common variant.\n")

	print "Checking to see how many interactions are impacted by variants in the probe binding site."

	fin_signif_int = open(args.input_interactions_file, 'rU')
	int_reader = csv.reader(fin_signif_int, delimiter="\t")

	fout_int = open(args.output_results, 'w')
	writer_int = csv.writer(fout_int, delimiter="\t")

	for interaction in int_reader:
		int_gene = interaction[0]

		if int_gene in genes_without_variants:
			interaction.append("No variants.")
			writer_int.writerow(interaction)

		elif int_gene in genes_with_common_variants:
			interaction.append("Common variants.")
			writer_int.writerow(interaction)

		elif int_gene in genes_without_common_variants:
			interaction.append("Rare variants only.")
			writer_int.writerow(interaction)

	fin_signif_int.close()
	fout_int.close()



if __name__ == '__main__':
	calc_frequency()