"""
This script identifies individuals who have rare variants in the probe binding site.
It then calls PLINK to remove these individuals.
"""

import csv
import argparse
import subprocess
import sys
import os

##################################################################################################
parser = argparse.ArgumentParser(description = "If ONLY rare variants are in the probe binding site, this script identifies the individuals with those variants and generates PED/MAP files without those people.")
parser.add_argument('-input_probes', '--input_probe_ids', help = "Provide a tab delimited list of probe_ids. The probe_id needs to be in the first column, and the gene_id in the 12 column."
					+ " Other columns will be ignored. A header is assumed. This is required.", 
					action='store', required = True)
parser.add_argument('-signif_interactions', '--input_interactions_file', help = "This needs to be the output from the variants_in_probes_check.py script. This is required.", 
					action='store', required = True)
parser.add_argument('-vcf_dir', '--input_vcf_directory', help = "Specify the directory that contains the vcf files used to run the INTERSNP analysis. This script assumes that the naming convention is dirctory_name/probeID.ped." +
					" This is required", 
					action='store', required = True)
parser.add_argument('-ped_dir', '--input_ped_directory', help = "Specify the directory that contains the ped/map files used to run the INTERSNP analysis. This script assumes that the naming convention is dirctory_name/probeID.ped." +
					" This is required", 
					action='store', required = True)
parser.add_argument('-out_dir', '--output_directory', help = "Specify the directory for output vcf/PLINK files. One file will be generated per probe. This is required", 
					action='store', required = True)
parser.add_argument('-out_results', '--output_results', help = "Specify the the name of the results file. This is required.", 
					action='store', required = True)

#Print error message if no input is provided.
if len(sys.argv)==1:
	parser.print_help()
	sys.exit(1)
args = parser.parse_args()

#recreate the command given
command = ""
for com in sys.argv:
	command = command + str(com) + " "

#create the logfile
logfile_name = args.output_results[:-4]+".log"
logfile = open(logfile_name, 'w')
logfile.write("You ran the command:\n")
logfile.write(command + "\n")

print "You ran the command:"
print command

#check input files can be found.
input_files = [args.input_probe_ids, args.input_interactions_file]
for input_file in input_files:
	if not os.path.isfile(input_file):
		logfile.write("Cannot find file " + str(input_file))
		parser.error("Cannot find file " + str(input_file))
	
############################################################################################

def ind_filter():

	fin_interactions = open(args.input_interactions_file, 'rU')
	int_reader = csv.reader(fin_interactions, delimiter="\t")


	genes_to_investigate = []

	for row in int_reader:
		probe_status = row[33]
		if probe_status == "Rare variants only.":  #check and see if we even need to do this analysis
			gene_id = row[0]
			genes_to_investigate.append(gene_id)
	fin_interactions.close()

	genes_to_investigate = set(genes_to_investigate)
	genes_to_investigate = list(genes_to_investigate)

	print "There are " + str(len(genes_to_investigate)) + " genes with only rare variants in the probe binding site."

	for gene in genes_to_investigate:
		
		#Get the probe_id for each gene
		probe_id = "NA" 
		fin_probe_ids = open(args.input_probe_ids, 'rU')		
		probe_reader = csv.reader(fin_probe_ids, delimiter="\t")
		for probe_row in probe_reader:
			probe_gene = probe_row[11]
			if probe_gene == gene:
				probe_id = probe_row[0]
		fin_probe_ids.close()

		#convert the VCF files to PLINK files
		vcf_file = args.input_vcf_directory + probe_id + ".recode.vcf"
		plink_out = args.output_directory + probe_id 
		query = "vcftools --vcf " + vcf_file + " --plink --out " + plink_out

		p = subprocess.Popen(query.split(' '),stdout=subprocess.PIPE, stderr=subprocess.PIPE)
		stdout, stderr = p.communicate()
		logfile.write("Converting VCF files to PED/MAP files with the following command:\n")
		logfile.write(query + "\n")
		logfile.write(stdout + "\n")
		logfile.write(stderr + "\n")

		#convert the PLINK PED/MAP files to raw files
		query = "plink --noweb --file " + plink_out + " --recodeAD --out " + plink_out + "_AD"
		p = subprocess.Popen(query.split(' '),stdout=subprocess.PIPE, stderr=subprocess.PIPE)
		stdout, stderr = p.communicate()
		logfile.write("Converting PED/MAP files to RAW files with the following command:\n")
		logfile.write(query + "\n")
		logfile.write(stdout + "\n")
		logfile.write(stderr + "\n")

		#This generates the list of individuals to exclude.
		fout_name = args.output_directory + probe_id + "_ind_to_exclude.txt"
		fout = open(fout_name, 'w')
		fout_writer = csv.writer(fout, delimiter = "\t")
		
		persons_with_rare_variants = []
		
		fin_ped_name =  plink_out + "_AD.raw"
		fin_ped = open(fin_ped_name, 'rU')
		fin_ped.readline()
		ped_reader = csv.reader(fin_ped, delimiter = " ")
		
		for person in ped_reader:
			genotype_positions = range(6,len(person))
			for position in genotype_positions:
				if person[position] != "0":
					persons_with_rare_variants.append(person[0])
		fin_ped.close()
		
		#Write out the list of people to exclude
		persons_with_rare_variants = list(set(persons_with_rare_variants))
		for person_rv in persons_with_rare_variants:
			results = [person_rv, person_rv]
			fout_writer.writerow(results)
		fout.close()

		#now, exclude those individuals.
		plink_fin = args.input_ped_directory + gene + "/" + gene + "_maf05_filtered_pheno"
		query = "plink --noweb --file " + plink_fin + " --remove " + fout_name
		query = query + " --recode --out " +  args.output_directory + gene + "_maf05_filtered_pheno_probe_filtered"             
		
		p = subprocess.Popen(query.split(' '),stdout=subprocess.PIPE, stderr=subprocess.PIPE)
		stdout, stderr = p.communicate()
		logfile.write("Converting PED/MAP files to RAW files with the following command:\n")
		logfile.write(query + "\n")
		logfile.write(stdout + "\n")
		logfile.write(stderr + "\n")




if __name__ == '__main__':
	ind_filter()
